/* CondInst.cpp
   Functions for a DLL library that can hang off of an MSI package. It registers/deregisters
   the TitraxConduit with the desktop system's HostSync manager.

   Copyright Jim Weller 2004

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA. 
*/

#include "TitraxInstall.h"


/*
*****************************************************************************
*
*     Function:      DllMain()
*
*     Description:   main entry point to the TitraxConduit component
*
*     Parameters(3):
*                    hInstance  - instance handle of the DLL
*                    dwReason   - why the entry point was called
*                    lpReserved - reserved
*
*     Returns:       1 if okay
*
*****************************************************************************
*/
extern "C" int APIENTRY DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{

    if (dwReason == DLL_PROCESS_ATTACH)
    {
        hLangInstance = hInstance;
    }
    return 1;   // ok
}


/*
******************************************************************************
* DLL Exported
*
*    Function: Install
*
*    Description: Function for the installer to call at end of install phase to do 
*                 register with HotSync.
*        
*
*    Parameters (1):
*        *st - a pointer to a string that can be set in visual studio
*        
*
*    Returns:
*        0 - success
*       !0 - error
*
******************************************************************************
*/
ExportFunc long Install(TCHAR *st){
    //result = InstallCond(    
    //            INSTALL_COND_DIR,                   //LPSTR szSupportDllDirectory, 
    //            INSTALL_COND_CREATOR,               //const TCHAR *pCreator,
    //            INSTALL_COND_DIR,                   //LPSTR szInstallDirectory, 
    //            INSTALL_COND_DLL,                   //const TCHAR *pConduit, 
    //            INSTALL_COND_DATA_DIR,              //const TCHAR *pDirectory,
    //            "",                                 //const TCHAR *pFile, we don't need to declare these really
    //            "",                                 //const TCHAR *pRemoteDB, we don't need to declare these really
    //            INSTALL_COND_NAME,                  //const TCHAR *pName,
    //            INSTALL_COND_PRI,                   //DWORD dwPriority, 
    //            INSTALL_COND_COMMENT);              //const TCHAR *pInfo );

    DWORD    dwReturnCode;
    BOOL  bHotSyncRunning = FALSE;

    // Prepare the full path of the conduit.
    // Applications should not place conduits in the Palm Desktop directory.
    // The Palm Desktop installer only manages the Palm Desktop conduits.
    TCHAR    szConduitPath[MAX_PATH];
    StringCchPrintf(szConduitPath,MAX_PATH,INSTALL_COND_DIR);
    StringCchCat(szConduitPath,MAX_PATH,CI_DIRECTORY_SEPARATOR_STR);
    StringCchCat(szConduitPath,MAX_PATH,INSTALL_COND_DLL);

    // Make sure the Conduit Managerdll exists
    struct _finddata_t dll_file;
    long hFile;
    if( (hFile = _findfirst( INSTALL_COND_DIR, &dll_file )) == -1L )
        return CI_ERR_CODE_CONDUIT_NOT_FOUND;
    
    // Load the Conduit Manager DLL.
    HINSTANCE hConduitManagerDLL;
    if( (dwReturnCode = LoadCondMgrDll(&hConduitManagerDLL, INSTALL_COND_DIR)) != 0 )
        return(dwReturnCode);
    
    // Prepare to install the conduit using Conduit Manager functions
    // get pointers to them from the DLL
    CmInstallCreatorPtr    lpfnCmInstallCreator;
    lpfnCmInstallCreator = (CmInstallCreatorPtr) GetProcAddress(hConduitManagerDLL, "CmInstallCreator");
    CmSetCreatorModulePtr    lpfnCmSetCreatorModule;
    lpfnCmSetCreatorModule = (CmSetCreatorModulePtr) GetProcAddress(hConduitManagerDLL, "CmSetCreatorModule");
    CmSetCreatorDirectoryPtr    lpfnCmSetCreatorDirectory;
    lpfnCmSetCreatorDirectory = (CmSetCreatorDirectoryPtr) GetProcAddress(hConduitManagerDLL, "CmSetCreatorDirectory");
    CmSetCreatorFilePtr    lpfnCmSetCreatorFile;
    lpfnCmSetCreatorFile = (CmSetCreatorFilePtr) GetProcAddress(hConduitManagerDLL, "CmSetCreatorFile");
    CmSetCreatorRemotePtr    lpfnCmSetCreatorRemote;
    lpfnCmSetCreatorRemote = (CmSetCreatorRemotePtr) GetProcAddress(hConduitManagerDLL, "CmSetCreatorRemote");
    CmSetCreatorNamePtr    lpfnCmSetCreatorName;
    lpfnCmSetCreatorName = (CmSetCreatorNamePtr) GetProcAddress(hConduitManagerDLL, "CmSetCreatorName");
    CmSetCreatorPriorityPtr    lpfnCmSetCreatorPriority;
    lpfnCmSetCreatorPriority = (CmSetCreatorPriorityPtr) GetProcAddress(hConduitManagerDLL, "CmSetCreatorPriority");
    CmSetCreatorInfoPtr    lpfnCmSetCreatorInfo;
    lpfnCmSetCreatorInfo = (CmSetCreatorInfoPtr) GetProcAddress(hConduitManagerDLL, "CmSetCreatorInfo");
    CmSetCreatorIntegratePtr    lpfnCmSetCreatorIntegrate;
    lpfnCmSetCreatorIntegrate = (CmSetCreatorIntegratePtr) GetProcAddress(hConduitManagerDLL, "CmSetCreatorIntegrate");
    
    // make sure our pointers are good
    if( (lpfnCmInstallCreator == NULL) 
        || (lpfnCmSetCreatorModule == NULL)
        || (lpfnCmSetCreatorDirectory == NULL)
        || (lpfnCmSetCreatorFile == NULL)
        || (lpfnCmSetCreatorRemote == NULL)
        || (lpfnCmSetCreatorName == NULL)
        || (lpfnCmSetCreatorPriority == NULL)
        || (lpfnCmSetCreatorInfo == NULL)
        || (lpfnCmSetCreatorIntegrate == NULL)
        )
    {
        // Return error code.
        return(CI_ERR_CODE_INVALID_CONDMGR);
    }
    
    // Get the Palm Desktop Installation directory
    TCHAR    szPalmDesktopDir[MAX_PATH];
    int    size=MAX_PATH;
    if( (dwReturnCode=GetPalmDirectory(hConduitManagerDLL, szPalmDesktopDir, &size)) != 0 )
        return dwReturnCode;
    
    // Load the HSAPI DLL.
    HINSTANCE hHsapiDLL;
    if( (dwReturnCode = LoadHsapiDll(&hHsapiDLL, szPalmDesktopDir, INSTALL_COND_DIR)) != 0 )
        return(dwReturnCode);
        
    // Shutdown the HotSync Process if it is running
    if( (bHotSyncRunning=HsRunning(hHsapiDLL)) )
    {
        // Check for any synchronizations in progress
        if( HsInProgress(hHsapiDLL) )
            return CI_ERR_CODE_HOTSYNC_IN_PROGRESS;
            
        HsStop(hHsapiDLL);
    }
    
    // Actually install the conduit as an Application Conduit
    dwReturnCode = (*lpfnCmInstallCreator)(INSTALL_COND_CREATOR, CONDUIT_APPLICATION);
    if( dwReturnCode == 0 )
    {
        dwReturnCode = (*lpfnCmSetCreatorName)(INSTALL_COND_CREATOR, szConduitPath);
        if( dwReturnCode != 0 ) return dwReturnCode;
        dwReturnCode = (*lpfnCmSetCreatorDirectory)(INSTALL_COND_CREATOR, INSTALL_COND_DATA_DIR);
        if( dwReturnCode != 0 ) return dwReturnCode;
        dwReturnCode = (*lpfnCmSetCreatorModule)(INSTALL_COND_CREATOR, INSTALL_COND_NAME);
        if( dwReturnCode != 0 ) return dwReturnCode;
        dwReturnCode = (*lpfnCmSetCreatorPriority)(INSTALL_COND_CREATOR, INSTALL_COND_PRI);
        if( dwReturnCode != 0 ) return dwReturnCode;
        dwReturnCode = (*lpfnCmSetCreatorInfo)(INSTALL_COND_CREATOR, INSTALL_COND_COMMENT);
        if( dwReturnCode != 0 ) return dwReturnCode;
        // Applications should always set Integrate to 0
        dwReturnCode = (*lpfnCmSetCreatorIntegrate)(INSTALL_COND_CREATOR, (DWORD)0);
        if( dwReturnCode != 0 ) return dwReturnCode;
    }
    
    // Re-start HotSync if it was running before
    if( bHotSyncRunning )
        HsStart(hHsapiDLL);
        
    CiErrMsgBox(dwReturnCode);
    return dwReturnCode;
}

/*
******************************************************************************
* DLL Exported
*
*    Function: Uninstall
*
*    Description: Function for the installer to call to do 
*                 deregister with HotSync.
*        
*
*    Parameters (1):
*        *st - a pointer to a string that can be set in visual studio
*        
*
*    Returns:
*        0 - success
*       OTHER - error
*
******************************************************************************
*/
ExportFunc long Uninstall(TCHAR *st)
{

    INT32    dwReturnCode;
    BOOL    bHotSyncRunning = FALSE;

    // Load the Conduit Manager DLL.
    HINSTANCE hConduitManagerDLL;
    if( (dwReturnCode = LoadCondMgrDll(&hConduitManagerDLL, INSTALL_COND_DIR)) != 0 )
        return(dwReturnCode);
    
    // Prepare to uninstall the conduit using Conduit Manager functions
    CmRemoveConduitByCreatorIDPtr    lpfnCmRemoveConduitByCreatorID;
    lpfnCmRemoveConduitByCreatorID = (CmRemoveConduitByCreatorIDPtr) GetProcAddress(hConduitManagerDLL, "CmRemoveConduitByCreatorID");
    
    if( (lpfnCmRemoveConduitByCreatorID == NULL) )
    {
        // Return error code.
        return(CI_ERR_CODE_INVALID_CONDMGR);
    }
    
    // Get the Palm Desktop Installation directory
    TCHAR    szPalmDesktopDir[MAX_PATH];
    int    size=MAX_PATH;
    if( (dwReturnCode=GetPalmDirectory(hConduitManagerDLL, szPalmDesktopDir, &size)) != 0 )
        return dwReturnCode;
    
    // Load the HSAPI DLL.
    HINSTANCE hHsapiDLL;
    if( (dwReturnCode = LoadHsapiDll(&hHsapiDLL, szPalmDesktopDir, INSTALL_COND_DIR)) != 0 )
        return(dwReturnCode);
        
    // Shutdown the HotSync Process if it is running
    if( (bHotSyncRunning=HsRunning(hHsapiDLL)) )
    {
        // Check for any synchronizations in progress
        if( HsInProgress(hHsapiDLL) )
            return CI_ERR_CODE_HOTSYNC_IN_PROGRESS;
            
        HsStop(hHsapiDLL);
    }
    
    // Actually uninstall the conduit
    dwReturnCode = (*lpfnCmRemoveConduitByCreatorID)(INSTALL_COND_CREATOR);
    
    // Re-start HotSync if it was running before
    if( bHotSyncRunning )
        HsStart(hHsapiDLL);

    CiErrMsgBox(dwReturnCode);

    if( dwReturnCode < 0 ) 
        return dwReturnCode;
    else 
        return(0);

}

/*
******************************************************************************
*
* Function:        LoadCondMgrDll()
*
* Description:    Loads the Conduit Manager DLL. 
*            1. Load the temporary CondManager.Dll from installer
*            2. Find the Palm Desktop installation directory.
*            3. Load the Palm Desktop Conduit Manager from their.
*            4. Return a handle to the Conduit Manager library.
*
* Parameters(2):
*        hCondMgrDll        - handle to instance of loaded Conduit Manager DLL.
*                            Returns NULL on error.
*        szInstallDir    - location of the Conduit Manager included with the
*                            installer.
*
* Returns:
*        0            - no error.
*        !0            - error code.
*
******************************************************************************
*/
INT32 LoadCondMgrDll(HINSTANCE* hCondMgrDll, LPSTR szInstallDir)
{
    INT32    dwReturnCode;
    HINSTANCE    hInstallerCondMgrDll;

    // Initialize the return value
    *hCondMgrDll=NULL;
    
    // First, load the temporary Conduit Manager Dll shipped with the installer
    TCHAR    szInstallerCondMgrPath[MAX_PATH];
    strncpy(szInstallerCondMgrPath, szInstallDir, MAX_PATH);
    strncat(szInstallerCondMgrPath, CI_DIRECTORY_SEPARATOR_STR, MAX_PATH-strlen(szInstallerCondMgrPath));
    strncat(szInstallerCondMgrPath, CI_CONDMGR_DLL_FILENAME, MAX_PATH-strlen(szInstallerCondMgrPath));
    if( (hInstallerCondMgrDll=LoadLibrary(szInstallerCondMgrPath)) == NULL )
        // Error loading temporary Conduit Manager Library
        return CI_ERR_CODE_USING_TEMP_CONDMGR;

    // Get the Palm Desktop Installation directory
    TCHAR    szPalmDesktopDir[MAX_PATH];
    int    size=MAX_PATH;
    if( (dwReturnCode=GetPalmDirectory(hInstallerCondMgrDll, szPalmDesktopDir, &size)) != 0 )
        return dwReturnCode;
            
    // Construct the path of the Palm Desktop Conduit Manager
    TCHAR    szPDCondMgrPath[MAX_PATH];
    strncpy(szPDCondMgrPath, szPalmDesktopDir, MAX_PATH);
    strncat(szPDCondMgrPath, CI_DIRECTORY_SEPARATOR_STR, MAX_PATH-strlen(szPDCondMgrPath));
    strncat(szPDCondMgrPath, CI_CONDMGR_DLL_FILENAME, MAX_PATH-strlen(szPDCondMgrPath));
            
    // Load the Conduit Manager library from the Palm Desktop directory
    if( (*hCondMgrDll=LoadLibrary(szPDCondMgrPath)) == NULL )
    {
        // Error loading Palm Desktop Conduit Manager library
        // use the one shipped with the installer
        *hCondMgrDll = hInstallerCondMgrDll;
    }
    else
    {
        // Success loading the Palm Desktop Conduit Manager
        // free the temporary Conduit Manager
        FreeLibrary(hInstallerCondMgrDll);
    }
    
    return 0;
}

/*
******************************************************************************
*
*    Function:        LoadHsapiDll()
*
*    Description:    Loads the HSAPI.DLL from the Palm Desktop installation
*        directory. If it does not exist there, load the one included in the
*        installer. If neither can be loaded, return an error.
*
*    Parameters(3):
*        hHsapiDLL                - returned handle to instance of loaded HSAPI DLL.
*                                   Returns NULL on error.
*        szPalmDesktopDirectory    - Palm Desktop installation directory.
*        szSupportDllDirectory    - location of the HSAPI Dll included with
*                                    the installer.
*
*    Returns:
*        0            - no error.
*        non-zero    - error code.
*
******************************************************************************
*/
INT32 LoadHsapiDll(HINSTANCE* hHsapiDLL, const TCHAR* szPalmDesktopDirectory,
                    const TCHAR* szSupportDllDirectory)
{
    // Initialize the return value
    *hHsapiDLL=NULL;
    
    // First, try loading the HSAPI Dll from the Palm Desktop directory
    TCHAR    szHsapiPath[MAX_PATH];
    strncpy(szHsapiPath, szPalmDesktopDirectory, MAX_PATH);
    strncat(szHsapiPath, CI_DIRECTORY_SEPARATOR_STR, MAX_PATH-strlen(szHsapiPath));
    strncat(szHsapiPath, CI_HSAPI_FILENAME, MAX_PATH-strlen(szHsapiPath));
    if( (*hHsapiDLL=LoadLibrary(szHsapiPath)) != NULL )
        // Successfully loaded HSAPI Library from Palm Desktop Directory
        return 0;

    // Second, try loading the HSAPI Dll included with the installer
    strncpy(szHsapiPath, szSupportDllDirectory, MAX_PATH);
    strncat(szHsapiPath, CI_DIRECTORY_SEPARATOR_STR, MAX_PATH-strlen(szHsapiPath));
    strncat(szHsapiPath, CI_HSAPI_FILENAME, MAX_PATH-strlen(szHsapiPath));
    if( (*hHsapiDLL=LoadLibrary(szHsapiPath)) != NULL )
        // Successfully loaded HSAPI Library from support directory
        return 0;
    
    // If we get here, then there was an error loading the library
    return CI_ERR_CODE_HSAPI_NOT_FOUND;
}




/*
******************************************************************************
* Function: GetPalmDirectory()
*
* Description: Determines the Palm Desktop install directory
*
* Parameters(3):
*        hCondMgrDll        - handle to instance of previously loaded Conduit
*                             Manager DLL.
*        pPDInstallDirectory    - buffer for storage of Palm Desktop directory.
*        pSize                - pointer to buffer size. On entry, size of buffer.
*                                On exit, actual length of path.
*
*    Returns:
*        0            - no error.
*        non-zero    - error code.
*
******************************************************************************
*/
INT32 GetPalmDirectory(HINSTANCE hCondMgrDll, TCHAR *pPDInstallDirectory,int    *pSize )
{
    INT32    dwReturnCode;

    // Get the Palm Desktop Installation directory
    CmGetHotSyncExecPathPtr    lpfnCmGetHotSyncExecPath;
    lpfnCmGetHotSyncExecPath = (CmGetHotSyncExecPathPtr) GetProcAddress(hCondMgrDll, "CmGetHotSyncExecPath");

    if( lpfnCmGetHotSyncExecPath == NULL )
        // Invalid Conduit Manager Library - missing functions
        return CI_ERR_CODE_INVALID_CONDMGR;
        
    if( (dwReturnCode=(*lpfnCmGetHotSyncExecPath)(pPDInstallDirectory, pSize)) != 0)
        // Error determining Palm Desktop install directory,
        // return the Conduit Manager error code.
        return dwReturnCode;

    // if '.exe' in the path
    if( strstr( pPDInstallDirectory, CI_EXECUTABLE_EXTENSION ) != NULL )
    {
        // Remove the "HotSync.exe" portion by walking
        // from the end of the string up to the 
        // directory separator
        TCHAR*    cPtr = &(pPDInstallDirectory[*pSize]);
        while( cPtr >= pPDInstallDirectory )
        {
            if( *cPtr == CI_DIRECTORY_SEPARATOR )
                break;
            cPtr--;
        }
        if( *cPtr == CI_DIRECTORY_SEPARATOR )
            *cPtr = '\0';
    }
    
    // Set the size of the returned path
    *pSize = strlen(pPDInstallDirectory);
            
    return 0;
}


/*
******************************************************************************
*
*    Function:        HsRunning()
*
*    Description:    Determines if HotSync is currently running.
*
*    Parameters(1):
*        hHsapiDLL        - handle to instance of loaded HSAPI DLL.
*
*    Returns:
*        TRUE    - HotSync running.
*        FALSE    - HotSync not running or HSAPI error.
*
******************************************************************************
*/
BOOL HsRunning(HINSTANCE hHsapiDLL)
{
    BOOL    bRetVal = FALSE;
    
    _ASSERT(hHsapiDLL != NULL);
    
    // Prepare to use the HSAPI functions
    HsCheckApiStatusPtr    lpfnHsCheckApiStatus;
    lpfnHsCheckApiStatus = (HsCheckApiStatusPtr) GetProcAddress(hHsapiDLL, "HsCheckApiStatus");
    
    if( lpfnHsCheckApiStatus )
    {
        if( (*lpfnHsCheckApiStatus)() == 0 )
            bRetVal = TRUE;
    }
    return bRetVal;
}


/*
******************************************************************************
*    Function:        HsInProgress()
*
*    Description:    Determines if there is currently a HotSync in progress.
*
*    Parameters(1):
*        hHsapiDLL        - handle to instance of loaded HSAPI DLL.
*
*    Returns:
*        TRUE    - HotSync in progress or HSAPI error.
*        FALSE    - no HotSync in progress.
*
******************************************************************************
*/
BOOL HsInProgress(HINSTANCE hHsapiDLL)
{
    BOOL    bRetVal = TRUE;
    DWORD    dwStatus;
    
    _ASSERT(hHsapiDLL);
    
    if(HsRunning(hHsapiDLL))
    {
        // Prepare to use the HSAPI functions
        HsGetSyncStatusPtr    lpfnHsGetSyncStatus;
        lpfnHsGetSyncStatus = (HsGetSyncStatusPtr) GetProcAddress(hHsapiDLL, "HsGetSyncStatus");
    
        if( lpfnHsGetSyncStatus )
        {
            if( (*lpfnHsGetSyncStatus)(&dwStatus) == 0 )
                if( dwStatus == HOTSYNC_STATUS_IDLE )
                    bRetVal = FALSE;
        }
    }
    return bRetVal;
}


/*
******************************************************************************
*    Function:        HsStop()
*
*    Description:    Shuts down HotSync. If HotSync isn't stopped after 
*                    a few seconds, just return.
*
*    Parameters(1):
*        hHsapiDLL        - handle to instance of loaded HSAPI DLL.
*
*    Returns:
*        void.
*
******************************************************************************
*/
VOID HsStop(HINSTANCE hHsapiDLL)
{
    _ASSERT(hHsapiDLL != NULL);
    BOOL    bHotSyncRunning=HsRunning(hHsapiDLL);
    
    if(bHotSyncRunning)
    {
        // Prepare to use the HSAPI functions
        HsSetAppStatusPtr    lpfnHsSetAppStatus;
        lpfnHsSetAppStatus = (HsSetAppStatusPtr) GetProcAddress(hHsapiDLL, "HsSetAppStatus");
    
        if( lpfnHsSetAppStatus )
            (*lpfnHsSetAppStatus)(HsCloseApp, HSFLAG_NONE);
        
        // Wait for HotSync to stop
        for( int i=0; (i<CI_HOTSYNC_MAX_WAIT) && bHotSyncRunning; i++ )
        {
            if( (bHotSyncRunning=HsRunning(hHsapiDLL)) == TRUE )
                Sleep(500);
        }
    }
}

/*
******************************************************************************
*
*    Function:       HsStart()
*
*    Description:    Starts HotSync. If HotSync doesn't start after a few
*                    seconds, just return.
*
*    Parameters(1):
*        hHsDLL - handle to instance of loaded HSAPI DLL.
*
*    Returns:
*        void.
*
******************************************************************************
*/
VOID HsStart(HINSTANCE hHsDLL)
{
    _ASSERT(hHsDLL);
    BOOL    bHotSyncRunning=HsRunning(hHsDLL);
    
    if(!bHotSyncRunning)
    {
        // Prepare to use the HSAPI functions
        HsSetAppStatusPtr    lpfnHsSetAppStatus;
        lpfnHsSetAppStatus = (HsSetAppStatusPtr) GetProcAddress(hHsDLL, "HsSetAppStatus");
    
        if( lpfnHsSetAppStatus )
            (*lpfnHsSetAppStatus)(HsStartApp, HSFLAG_NONE);
        
        // Wait for HotSync to start
        for( int i=0; (i<CI_HOTSYNC_MAX_WAIT) && !bHotSyncRunning; i++ )
        {
            if( (bHotSyncRunning=HsRunning(hHsDLL)) == FALSE )
                Sleep(500);
        }
    }
}


/*
******************************************************************************
*
*    Function:        CiErrMsgBox()
*
*    Description:
*        Displays a Message Box giving the error code & text from string resources.
*
*    Parameters (1):
*
*        dwResult        - the result of the InstallCond.
*
*    Returns:
*        void.
*
******************************************************************************
*/
void CiErrMsgBox(DWORD dwResult)
{
    TCHAR error_string[MAX_PATH];

    switch( dwResult )
    {
        case    0:
            // no error. just return.
            return;
     
        case    CI_ERR_CODE_INVALID_CONDMGR:
            StringCchPrintf(error_string,MAX_PATH,resString(IDS_CI_ERR_CODE_INVALID_CONDMGR));
            break;
        
        case    CI_ERR_CODE_HSAPI_NOT_FOUND:
            StringCchPrintf(error_string,MAX_PATH,resString(IDS_CI_ERR_CODE_HSAPI_NOT_FOUND));
            break;
        
        case    CI_ERR_CODE_USING_TEMP_CONDMGR:
            StringCchPrintf(error_string,MAX_PATH,resString(IDS_CI_ERR_CODE_USING_TEMP_CONDMGR));
            break;
        
        case    CI_ERR_CODE_HOTSYNC_IN_PROGRESS:
            StringCchPrintf(error_string,MAX_PATH,resString(IDS_CI_ERR_CODE_HOTSYNC_IN_PROGRESS));
            break;
        
        case    ERR_CONDUIT_MGR:
            StringCchPrintf(error_string,MAX_PATH,resString(IDS_CI_ERR_CONDUIT_MGR));
            break;
        
        case    CI_ERR_CODE_CONDUIT_NOT_FOUND:
            StringCchPrintf(error_string,MAX_PATH,resString(IDS_CI_ERR_CODE_CONDUIT_NOT_FOUND));
            break;

        case    ERR_INDEX_OUT_OF_RANGE:
            StringCchPrintf(error_string,MAX_PATH,resString(IDS_CI_ERR_INDEX_OUT_OF_RANGE));
            break;
        
        case    ERR_UNABLE_TO_DELETE:
            StringCchPrintf(error_string,MAX_PATH,resString(IDS_CI_ERR_UNABLE_TO_DELETE));
            break;
        
        case    ERR_NO_CONDUIT:
            StringCchPrintf(error_string,MAX_PATH,resString(IDS_CI_ERR_NO_CONDUIT));
            break;
        
        case    ERR_NO_MEMORY:
            StringCchPrintf(error_string,MAX_PATH,resString(IDS_CI_ERR_NO_MEMORY));
            break;
        
        case    ERR_CREATORID_ALREADY_IN_USE:
            StringCchPrintf(error_string,MAX_PATH,resString(IDS_CI_ERR_CREATORID_ALREADY_IN_USE));
            break;
        
        case    ERR_REGISTRY_ACCESS:
            StringCchPrintf(error_string,MAX_PATH,resString(IDS_CI_ERR_REGISTRY_ACCESS));
            break;
        
        case    ERR_UNABLE_TO_CREATE_CONDUIT:
            StringCchPrintf(error_string,MAX_PATH,resString(IDS_CI_ERR_UNABLE_TO_CREATE_CONDUIT));
            break;
        
        case    ERR_UNABLE_TO_SET_CONDUIT_VALUE:
            StringCchPrintf(error_string,MAX_PATH,resString(IDS_CI_ERR_UNABLE_TO_SET_CONDUIT_VALUE));
            break;
        
        case    ERR_INVALID_HANDLE:
            StringCchPrintf(error_string,MAX_PATH,resString(IDS_CI_ERR_INVALID_HANDLE));
            break;
        
        case    ERR_BUFFER_TOO_SMALL:
            StringCchPrintf(error_string,MAX_PATH,resString(IDS_CI_ERR_BUFFER_TOO_SMALL));
            break;
        
        case    ERR_VALUE_NOT_FOUND:
            StringCchPrintf(error_string,MAX_PATH,resString(IDS_CI_ERR_VALUE_NOT_FOUND));
            break;
        
        case    ERR_INVALID_CREATOR_ID:
            StringCchPrintf(error_string,MAX_PATH,resString(IDS_CI_ERR_INVALID_CREATOR_ID));
            break;
        
        case    ERR_INVALID_POINTER:
            StringCchPrintf(error_string,MAX_PATH,resString(IDS_CI_ERR_INVALID_POINTER));
            break;
        
        case    ERR_UNABLE_TO_INSTALL_OLD:
            StringCchPrintf(error_string,MAX_PATH,resString(IDS_CI_ERR_UNABLE_TO_INSTALL_OLD));
            break;
        
        case    ERR_INVALID_CONDUIT_TYPE:
            StringCchPrintf(error_string,MAX_PATH,resString(IDS_CI_ERR_INVALID_CONDUIT_TYPE));
            break;
        
        case    ERR_INVALID_COM_PORT_TYPE:
            StringCchPrintf(error_string,MAX_PATH,resString(IDS_CI_ERR_INVALID_COM_PORT_TYPE));
            break;
        
        case    ERR_NO_LONGER_SUPPORTED:
            StringCchPrintf(error_string,MAX_PATH,resString(IDS_CI_ERR_NO_LONGER_SUPPORTED));
            break;
        
        default:
            StringCchPrintf(error_string,MAX_PATH,resString(IDS_CI_ERR_DEFAULT));
            break;
    }
    MessageBox(GetFocus(), error_string, THIS_DLL, MB_OK | MB_ICONEXCLAMATION );

}


/*
*****************************************************************************
*
*     Method:        resString
*
*     Description:    Convert resourceId to string pointer.
*                     return a pointer to a static buffer that has the string from the resource id in it.
*                     NOTE: STATIC BUFFER. Be quick about it and only do copies. Don't deal in pointers.
*
*     Parameters(1): UINT stringResID - the id number of the string resource
*
*     Returns: 
*              TCHAR* - pointer to the resource string in a static buffer
*
*****************************************************************************
*/
TCHAR* resString( UINT stringResId )
{
    static TCHAR resStringBuffer[MAX_PATH];
    LoadString((HINSTANCE)hLangInstance, stringResId, resStringBuffer, MAX_PATH);
    return resStringBuffer;
}


