/* CondInst.h
   Header for a DLL library that can hang off of an MSI package. It registers/deregisters
   the TitraxConduit with the desktop system's HostSync manager.

   Copyright Jim Weller 2004

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA. 
*/

#ifndef  __TITRAX_INSTALL__
#define  __TITRAX_INSTALL__

#include <windows.h>         // windows
#include <io.h>              // IO
#include <crtdbg.h>          // Terminal debugging
#include <strsafe.h>         // new buffer secure microsoft string functions

#include <CondMgr.h>         // Palm CDK: Conduit Manager
#include <Hsapi.h>           // Palm CDK: HotSync API

#include "TitraxInstall_RC.h"

/* windows garbage to make a function available in the DLL */
#ifndef ExportFunc
#define ExportFunc __declspec( dllexport )
#endif
#ifndef ImportFunc
#define ImportFunc __declspec( dllimport )
#endif

/* standard handles for DLLs*/
HANDLE hLangInstance;
extern HANDLE hLangInstance;


/* name of output DLL. Used for error message dialogs.  */
#define    THIS_DLL "TitraxConduit.dll"

/* Parms for conduit installation. Need to sync up w/ visual studio. 
   I'll delve some type of macros later. */
#define INSTALL_COND_DIR "C:\\program files\\Sleepless"
#define INSTALL_COND_CREATOR "Ttrx"
#define INSTALL_COND_DLL "TitraxConduit.dll"
#define INSTALL_COND_DATA_DIR "Titrax"
#define INSTALL_COND_NAME "Titrax"
#define INSTALL_COND_PRI 2
#define INSTALL_COND_COMMENT "DLL Installled"

/* other DLL's we'll check for along the way */
#define    CI_CONDMGR_DLL_FILENAME   "CondMgr.dll"
#define    CI_HSAPI_FILENAME        "HsApi.dll"

/* for paths */
#define    CI_DIRECTORY_SEPARATOR        '\\'  // single TCHAR (a backslash)
#define    CI_DIRECTORY_SEPARATOR_STR    "\\"  // 

#define    CI_EXECUTABLE_EXTENSION    ".exe"
#define    CI_HOTSYNC_MAX_WAIT        60            // wait for HotSync to start/stop in seconds

/* Error codes returned by the conduit mangling functions */
#define    CI_ERR_CODE_BASE                    -2400
#define    CI_ERR_CODE_USING_TEMP_CONDMGR       (CI_ERR_CODE_BASE-0x01)
#define    CI_ERR_CODE_INVALID_CONDMGR          (CI_ERR_CODE_BASE-0x02)
#define    CI_ERR_CODE_HSAPI_NOT_FOUND          (CI_ERR_CODE_BASE-0x03)
#define    CI_ERR_CODE_CONDUIT_NOT_FOUND        (CI_ERR_CODE_BASE-0x04)
#define    CI_ERR_CODE_HOTSYNC_IN_PROGRESS      (CI_ERR_CODE_BASE-0x05)

extern "C" {
    /* for un/installing the conduit */
    /* callbacks. These get called by the MSI installer
       to handle register/deregister with HotSync during installation/deinstallation*/
    ExportFunc long Install(TCHAR *);
    ExportFunc long Uninstall(TCHAR *);
}


/* internal utility methods included in this DLL but not exported .
   Most of these take advantage of the CDK API's (Sync Manager, Conduit manager, et al.) */
INT32 GetPalmDirectory(HINSTANCE , TCHAR *,int    * );
INT32 LoadCondMgrDll(HINSTANCE* , LPSTR );
INT32 LoadHsapiDll(HINSTANCE* , const TCHAR* ,  const TCHAR* );
BOOL HsRunning(HINSTANCE );
BOOL HsInProgress(HINSTANCE );
VOID HsStop(HINSTANCE );
VOID HsStart(HINSTANCE );
void CiErrMsgBox(DWORD );
TCHAR* resString( UINT  );


/* 
Declare functions we'll use from Palm's  DLLs. We must declare 
pointers to their external functions. It would have been nicer if they had 
dllexport'ed them so that we could dllimport without all the pointer crap.
Maybe they did it for portability?

    "If you declare a function or object without the dllimport or dllexport attribute,
    the function or object is not considered part of the DLL interface. Therefore, 
    the definition of the function or object must be present in that module or in 
    another module of the same program. [...]"
    ms-help://MS.VSCC.2003/MS.MSDNQTR.2003FEB.1033/vclang/html/_pluslang_general_rules_and_limitations.htm

*/

// Conduit Manager (CondMgr.dll)
typedef int (WINAPI *CmGetCorePathPtr)(TCHAR *pPath, int *piSize);
typedef int (WINAPI *CmGetHotSyncExecPathPtr)(TCHAR *szPath, int *iSize);
typedef int (WINAPI *CmInstallCreatorPtr)(const TCHAR *pCreator, int iType);
typedef int (WINAPI *CmRemoveConduitByCreatorIDPtr)(const TCHAR *pCreator);
typedef int (WINAPI *CmSetCreatorModulePtr)(const TCHAR *pCreator, const TCHAR *pConduitModule);
typedef int (WINAPI *CmSetCreatorDirectoryPtr)(const TCHAR *pCreator, const TCHAR *pDirectory);
typedef int (WINAPI *CmSetCreatorFilePtr)(const TCHAR *pCreator, const TCHAR *pFile);
typedef int (WINAPI *CmSetCreatorRemotePtr)(const TCHAR *pCreator, const TCHAR *pRemote);
typedef int (WINAPI *CmSetCreatorNamePtr)(const TCHAR *pCreator, const TCHAR *pConduitName);
typedef int (WINAPI *CmSetCreatorNamePtr)(const TCHAR *pCreator, const TCHAR *pConduitName);
typedef int (WINAPI *CmSetCreatorPriorityPtr)(const TCHAR *pCreator, DWORD dwPriority);
typedef int (WINAPI *CmSetCreatorInfoPtr)(const TCHAR *pCreator, const TCHAR *pInfo);
typedef int (WINAPI *CmSetCreatorIntegratePtr)(const TCHAR *pCreator, DWORD dwIntegrate);
typedef int (WINAPI *CmSetCreatorValueDwordPtr)(const TCHAR *pCreator, TCHAR *pValue, DWORD dwValue);

// HotSync API (HSAPI.dll)
typedef int (WINAPI *HsCheckApiStatusPtr)(void);
typedef int (WINAPI *HsGetSyncStatusPtr)(DWORD *dwStatus);
typedef int (WINAPI *HsSetAppStatusPtr)(HsStatusType statusType, DWORD dwStartFlags);

#endif  // __TITRAX_INSTALL__